@@ -19,11 +19,13 @@ class Initializer(object):
 			self.prefix_meta = str(Path.home())+'/.stack'+self.storage.dataset
 			self.prefix_curr = str(Path.home())+'/.stack'+self.storage.dataset+'current/'
 			self.prefix_commit = str(Path.home())+'/.stack'+self.storage.dataset+'commits/'
+			self.prefix_history = str(Path.home())+'/.stack'+self.storage.dataset+'history/'
 			self.prefix_diffs = str(Path.home())+'/.stack'+self.storage.dataset+'diffs/'
 		else:
 			self.prefix_meta = '.stack/'+self.storage.dataset
 			self.prefix_curr = '.stack/'+self.storage.dataset+'current/'
 			self.prefix_commit = '.stack/'+self.storage.dataset+'commits/'
+			self.prefix_history = '.stack/'+self.storage.dataset+'history/'
 			self.prefix_diffs = '.stack/'+self.storage.dataset+'diffs/'
 
 		self.start_check()
@@ -42,7 +44,7 @@ class Initializer(object):
 	def start_check(self):
 		# checks if the dataset exists
 		if not self.verify_setup():
-			print('no .stack/ folder found! creating one...')
+			print('no .stack/ subfolder found! creating one...')
 			self.setupDataset()
 		return True
 
@@ -90,9 +92,10 @@ class Initializer(object):
 		self.storage.resetBuffer()
 
 		# saves the metadata of the backup
+		print('loading dataset list')
 		metapath = self.prefix_meta + 'current.json'
-
 		keys, lm = self.storage.loadDatasetList()
+		print(lm)
 		current = {'keys': keys, 'lm': lm}
 		self.storage.addFileFromBinaryGlobal(metapath,io.BytesIO(json.dumps(current).encode('ascii')))
 		self.storage.resetBuffer()
@@ -114,7 +117,7 @@ class Initializer(object):
 
 		# saves the metadata of the backup
 		metapath = self.prefix_meta + 'current.json'
-
+		print('loading dataset list')
 		keys, lm = self.storage.loadDatasetList()
 		current = {'keys': keys, 'lm': lm}
 		self.storage.addFileFromBinaryGlobal(metapath,io.BytesIO(json.dumps(current).encode('ascii')))
@@ -191,8 +194,17 @@ class Initializer(object):
 				'comment' : 'added '+ file['key'],
 			}
 
+			history = {}
+			history[1] = commit
 			commitpath = self.prefix_commit + file['key'] + '/' + str(1).zfill(10)
 			self.storage.addFileFromBinaryGlobal(commitpath,io.BytesIO(json.dumps(commit).encode('ascii')))
+			
+			self.storage.resetBuffer()
+			
+			histpath = self.prefix_history + file['key'] + '/history.json'
+			self.storage.addFileFromBinaryGlobal(histpath,io.BytesIO(json.dumps(history).encode('ascii')))
+			self.storage.resetBuffer()
+
 		self.storage.resetBuffer()
 		
 		return True
